      program oab
c
c     main calling program for the 2 level CRD 
c     with background continuum problem of 
c     Olson, Auer Buchler 1986 JQSRT 35, 431
c
      include 'prec'
      parameter (ndep=70, nfrq=30, nmu=3)
      dimension tau(ndep), s(ndep), epsilon(ndep), b(ndep),
     * pe(ndep), s1(ndep), starl(ndep),jbar(ndep),
     * s2(ndep),s3(ndep),s4(ndep),sng(ndep),wt(ndep),t(ndep)
      dimension p(ndep), pms(ndep), iplus(0:ndep),iminus(0:ndep)
      real*8 iplus,iminus,jbar
      dimension x(0:nfrq), phi(0:nfrq), wphi(0:nfrq)
      dimension xmu(nmu), wmu(nmu)
      logical ngon
      ngon=.true.
C      ngon=.false.
      iout=1
      crit=1.e-10
      itmax=400
      itran=0
c
c depth grid, constant d log tau is used.
c      
      taumin=0.001
      taumax=1.e8
      dlgtau=log10(taumax/taumin)/float(ndep-1)
      dtau=10.**dlgtau
      tau(1)=taumin
      do 100 i=2,ndep
         tau(i)=tau(i-1)*dtau
 100  continue
c
c  epsilon and B parameters
c      
      if(iout .eq. 1) write(6,*)' I  Tau        Epsilon    B'
      do 101 i=1,ndep
         epsilon(i)=1.e-4
         b(i)=1.
c         if(tau(i) .gt. 1. .and. tau(i) .lt. 100.) b(i)= 3.
         if(iout .eq. 1) write(6,1000) i, tau(i),epsilon(i),b(i)
 101  continue
 1000 format(i3, 3(1x,1p,e10.3))
c
c  line profile, assumed the same at each depth
c  trapezoidal weights, multiplied by two for a one-sided profile
c
      ndw=4
      ptot=0.
      dx=1./float(nfrq)*float(ndw)
      do 102 i=0, nfrq
         x(i)=float(i)*dx
         phi(i)=exp(-(x(i)*x(i)))
         wphi(i)= 2.0*dx
         if(i .eq. 0 .or. i .eq. nfrq) wphi(i)=wphi(i)/2.
         ptot=ptot+phi(i)*wphi(i)
 102  continue
c
c Normalize so integral is unity 
c
      do 1021 i=0,nfrq
         phi(i)=phi(i)/ptot
 1021 continue
      tmin=phi(nfrq)*tau(ndep)
      if(iout .eq. 1) then 
         write(6,*)' I  X          Phi        Wphi'
         do 104 i=0, nfrq
            write(6,1000) i, x(i),phi(i), wphi(i)
 104     continue
      endif
      if(tmin .lt. 1.0) then
         write(6,*) 'Atmosphere is too thin: tau max =', tmin
         stop
      else 
         write(6,*) 'Base optical depth in wing =',tmin
      endif
c
c  Gaussian quadrature in angle
c
      x1=0.
      x2=1.
      call gausi(3,x1,x2,wmu,xmu)
      if(iout .eq. 1) write(6,*)' I  Mu           Xmu'
      do 103 i=1, nmu
         if(iout .eq. 1) write(6,1001) i, xmu(i),wmu(i)
 103  continue
 1001 format(i3, 2(1x,1p,e10.3))
c
c  Starting approximation, eq (16) of OAB, AND
c  computation of Lambda* (variable starl), eqs 19 and 20 of OAB
c
      if(iout .eq. 1) write(6,*)
     * ' I  tau        p          1/(1+tau)   S(start)    S(start)/B
     * Lambda*'  
      do 107 k=1,ndep
         pe(k)=0.
         starl(k)=0.
         do 105 i=0,nfrq
            uistar=0.
            do 106 j=1,nmu
               pe(k)=pe(k)+wmu(j)*wphi(i)*phi(i)*
     *              exp(-phi(i)*tau(k)/xmu(j))
c
c  construct lambda star:
c  this is eq 31 of Hubeny 1992 (Feautrier form)
c
c  include approximate boundary values for lambda* (k =1, k=ndep)
c
               if(k .eq. 1) then 
                  dtdm1=(tau(k)-0.)*phi(i)/xmu(j)
               else
                  dtdm1=(tau(k)-tau(k-1))*phi(i)/xmu(j)
               endif
               if(k .eq. ndep) then 
                  dtd=dtdm1
               else
                  dtd=(tau(k+1)-tau(k))*phi(i)/xmu(j)
               endif
               dt=(dtd+dtdm1)/2.
               t1=(1. - exp(-dtd))/dtd
               t2=(1. - exp(-dtdm1))/dtdm1
C OAB BEGIN  **************************************************
               uistar= 1. / (1. + (t1/dt + t2/dt))
               starl(k)=starl(k) + uistar*phi(i)*wphi(i)*wmu(j)
 106        continue
C OAB END    **************************************************
C OK BEGIN   **************************************************
C               uistar = uistar + (t1+t2)*wmu(j)
C 106        continue
C            starl(k) = starl(k) +(1.-0.5*uistar)*phi(i)*wphi(i)
C OK END     **************************************************
 105     continue
c eq (16)
c         s1(k)=epsilon(k)*b(k)/ (1. - (1.-epsilon(k))*(1.-pe(k)))
c eq (29) of Scharmer 1981
         s1(k)=b(k)*sqrt(epsilon(k)/ (epsilon(k)+2.*pe(k)))
c         s1(k)=b(k)
         s(k)=s1(k)
         if(iout .eq. 1) write(6,1002) k, tau(k),pe(k),
     *   1./(1.+tau(k)),s1(k),s1(k)/b(k), starl(k)
 107  continue
 1002 format(i3, 6(1x,1p,e10.3))
c
c  main loop:
c  Now compute RHS of 22, get formal solution and update source fn.
c      
c      goto 998
      kount=0
 999  continue
      kount=kount+1
      dsmax=0.
      do 112 k=1,ndep
         jbar(k)=0.
 112  continue
c solve transfer problem and compute jbar:
      do 110 j=1,nmu
         xmy=xmu(j)
c
c  If incoming radiation use itran ge 10
c
c         iminus(0)=2.
c         itran=10
         iminus(0)=0.
         itran=0
         do 111 i=0,nfrq
C begin June 14, 2000
            do 1131 k=1,ndep
               t(k)=tau(k)*phi(i)
 1131       continue
            call transp(xmy,ndep,t,s,itran,p,pms,iplus,iminus)
C           call transp(xmy,ndep,tau,s,itran,p,pms,iplus,iminus)
C end June 14, 2000
            do 113 k=1,ndep
               jbar(k)=jbar(k)+p(k)*phi(i)*wphi(i)*wmu(j)
 113        continue
 111     continue
 110  continue
c      if(iout .eq. 1) write(6,*)
c     * ' I sold       jbar    starl*sold     snew'
      do 114 k=1,ndep
         s4(k)=s3(k)
         s3(k)=s2(k)
         s2(k)=s1(k)
         rhs= (1.-epsilon(k))* (jbar(k) -starl(k)*s1(k)) + 
     *        epsilon(k)*b(k)
         s(k)=rhs / (1. - (1.-epsilon(k))*starl(k))
         ds=(s(k)-s1(k))/s1(k)
c         if(iout .eq. 1) write(6,1002) k, s1(k),jbar(k), 
c     *                   starl(k)*s1(k),s(k)
         s1(k)=s(k)
         wt(k)=1./jbar(k)
         dsmax = max(dsmax,abs(ds))
 114  continue
      write(6,*) 'Iteration ',kount,' max relative change in s is',dsmax
      if (ngon .and. kount .ge. 4 .and. mod(kount,3) .eq. 0) then 
         write(6,*)'NG acceleration'
         call ng(ndep,wt,s,s2,s3,s4)
         do 117 k=1,ndep
            s(k)=s4(k)
            s1(k)=s(k)
 117     continue
      endif
      if(kount .ge. itmax) stop
      if(dsmax .gt. crit) goto 999
      write(6,*) 'SOLUTION:'
      write(6,*) 'depth tau        B          S '
      do 115 k=1,ndep-1
         write(*,1002)k,tau(k), b(k), s(k)
 115  continue
 998  continue
      stop
      end
C
C*********************************************************************
C
C
C  Note from Phil to Students:
C
C  This might even be self explanatory: but there are some conventions
C  to follow
C
C  1.  Let your depth grid start at point 1 to ndep: 1 is the outermost
C      point.  This is important since the boundary conditions are 
C      written with this explicitly in mind.
C  2.  Should you want to include incoming radiation, use IMINUS(0)=...
C      and a suitable choice of ITRAN
C  3.  The variable P that is output here is variable u in the Olson 
C      et al paper. 
C
C  4.  You should just call the routine TRANSP to get a formal solution
C      (it calls the routine TRANF itself).
C
C  5.  Yes, Daniel found the compilation error in tranf.f -
C      this has been fixed May 11, 2000
C  FYI:
C  My calling program that solves the OAB problem based on the attached 
C  routines contains a total of 180 lines of code- 
C
C  the actual "guts" of the thing
C  is less than 100 lines of code
C
C*********************************************************************
C
      SUBROUTINE TRANSP(XMY,NDEP,TAU,S,ITRAN,
     *                  P,PMS,IPLUS,IMINUS)
C
C  SOLVES THE RADIATIVE TRANSFER EQUATION WITH GIVEN SOURCE FUNCTION.
C  SOLVES FOR P-S WHERE P IS FEAUTRIERS P; I.E.
C
C    P = 0.5*(I(XMU)+I(-XMU))
C
C  IN THE PRESENCE OF VELOCITY FIELDS:
C
C    P = 0.5*(I(NY,XMU) + I(-NY,-XMU))
C
C  ITRAN DETERMINES THE MODE OF FORMAL SOLUTION
C  ITRAN  = 0  FEAUTRIER SOLUTION
C           1  FEAUTRIER SOLUTION TO CUBIC SPLINE ACCURACY, REF:
C              KUNASZ, HUMMER, 1974, MNRAS 166,19
C              MIHALAS, 1974, APJ SUPPL 28,343
C           2  FEAUTRIER SOLUTION HERMITE, REF:
C              AUER, 1976, JQSRT 16,931
C
C  VARIABLES:
C
C  XMY   : ANGULAR COSINE                                       (IN)
C  NDEP  : NUMBER OF DEPTH POINTS                               (IN)
C  TAU   : MONOCHROMATIC OPTICAL DEPTH SCALE                    (IN)
C  S     : MONOCHROMATIC SOURCE FUNCTION                        (IN)
C  ITRAN : DETERMINES MODE OF FORMAL SOLUTION                   (IN)
C
C  P     : MEAN BIDIRECTIONAL INTENSITY (CF. ABOVE)            (OUT)
C  PMS   : P-S                                                 (OUT)
C  IPLUS : RADIATION INTENSITY, OUTGOING RAYS                  (OUT)
C  IMINUS: RADIATION INTENSITY, INGOING RAYS                   (IN/OUT)
C
C:
C: TRANSP 86-09-04  MODIFICATIONS: (GORAN SCHARMER, MATS CARLSSON)
C:        INCLUDES CALL TO CUBIC SPLINE INTEGRAL FORMAL SOLVER
C:
C:        92-10-08  MODIFICATIONS: (MATS CARLSSON)
C:        ITRAN=10-14 ALLOWED (ITRAN.GT.10 INCLUDES INCOMING RADIATION)
C:
      include 'prec'
      parameter (mdep=200)
      dimension tau(ndep), s(ndep)
      dimension p(ndep), pms(ndep), iplus(0:ndep),iminus(0:ndep)
      REAL*8 IPLUS,IMINUS
      integer ITRAN
      dimension TAUQ(0:MDEP),DTAUQ(MDEP),A1(MDEP),C1(MDEP)
C
C
C 
      IF(NDEP .GT. MDEP) THEN
         WRITE(6,*)' YOU MUST INCREASE MDEP TO BE .GE. NDEP'
         STOP
      ENDIF
C
C K=1: UPPER BOUNDARY
C
      CMU=0.5/XMY
      DTAUQ(2)=(2.)*(TAU(2)-TAU(1))*CMU
      A1(1)=1./DTAUQ(2)
      T=TAU(1)*1.0*2.0*CMU
      TAUQ(1)=T
      DTAUQ(1)=T
C
C  CALCULATE DTAUQ 
C
      DO 100 K=2,NDEP
        DTAUQ(K)=2.0*(TAU(K)-TAU(K-1))*CMU
  100 CONTINUE
C
C  CALCULATE TAUQ
C
      DO 110 K=2,NDEP
        TAUQ(K)=TAUQ(K-1)+DTAUQ(K)
  110 CONTINUE
C
C  CALCULATE A1 AND C1
C
      DO 120 K=2,NDEP-1
        A1(K)=2./(DTAUQ(K)+DTAUQ(K+1))/DTAUQ(K)
        C1(K)=2./(DTAUQ(K)+DTAUQ(K+1))/DTAUQ(K+1)
  120 CONTINUE
C
C  CHOOSE THE METHOD OF FORMAL SOLUTION
C
      IF(MOD(ITRAN,10).LE.2) THEN
        CALL TRANF(itran,ndep,tauq,dtauq,s,a1,c1,p,iplus,iminus)
      ELSE
        write(6,*)'MOD(ITRAN,10) MUST BE 1,2,11, or 12'
        stop
      ENDIF
C
C CALCULATE P(K)-S(K)
C
      PMS(1)=P(1)-S(1)
      DO 130 K=2,NDEP-1
        IF(ABS(A1(K)).GT.1.0) THEN
          PMS(K)=P(K)-S(K)
        ELSE
          PMS(K)=C1(K)*(P(K+1)-P(K))-A1(K)*(P(K)-P(K-1))
        END IF
  130 CONTINUE
      PMS(NDEP)=(P(NDEP-1)-P(NDEP)+S(NDEP)-S(NDEP-1))
     * /(DTAUQ(NDEP)+0.5*DTAUQ(NDEP)**2)
C
c      do 150  k=1,ndep
c         write(6,*) k,'P,pms ',p(k),pms(k)
c 150  continue
      RETURN
      END
C
C**********************************************************************
C
      SUBROUTINE TRANF(itran,ndep,tauq,dtauq,s,a1,c1,p,iplus,iminus)
C
C  FORMAL SOLVER FOR THE TRANSFER EQUATION USING FEAUTRIER TECHNIQUE
C
C  MOD(ITRAN,10) DETERMINES THE MODE OF FORMAL SOLUTION
C        =  0  FEAUTRIER SOLUTION
C           1  FEAUTRIER SOLUTION TO CUBIC SPLINE ACCURACY, REF:
C              AUER, 1976, JQSRT 16,931
C           2  FEAUTRIER SOLUTION HERMITE
C
C  THE  UPPER AND LOWER  BOUNDARY CONDITIONS ARE DERIVED FROM SECOND
C  ORDER TAYLOR EXPANSIONS, WITH ESTIMATES OF THE INCIDENT RADIATION
C  FIELDS:
C
C    P(2)    =  P(1) + DTAU*P'(1) + 0.5*DTAU**2*P''(1)
C    P'(1)   =  P(1) - I(-XMU)
C    I(-XMU) =  S(1)*(1.-EXP(-TAUQ(1)))
C
C    P(N-1)  =  P(N) - DTAU*P'(N) + 0.5*DTAU**2*P''(N)
C    P'(N)   =  I(XMU) - P(N)
C    I(XMU)  =  S(N) + (S(N)-S(N-1))/DTAU
C
C  STEINS  TRICK
C  (STORING THE SUM OF ELEMENTS INSTEAD OF THE DIAGONAL) IS USED  TO
C  AVOID  NUMERICAL  ROUND-OFF  PROBLEMS  AT  SMALL  OPTICAL DEPTHS.
C
C  CODED BY: A.NORDLUND (OCT-1981). REVISED (MAR-1982).
C
C: TRANF  92-10-08  MODIFICATIONS: (MATS CARLSSON)
C:        INCOMING RADIATION FIELD IS SPECIFIED IN IMINUS(0)
C:        THIS IS INDICATED BY ITRAN.GT.10
C:
      include 'prec'
      parameter (mdep=200)
      dimension s(ndep)
      dimension p(ndep), iplus(0:ndep),iminus(0:ndep)
      REAL*8 IPLUS,IMINUS
      integer ITRAN
      dimension TAUQ(0:MDEP),DTAUQ(MDEP),A1(MDEP),C1(MDEP)
C
      DIMENSION SP1(MDEP),SP2(MDEP),SP3(MDEP)
C
      T=TAUQ(1)
      SP1(1)=0.
      SP2(1)=1.+2.*A1(1)
      SP3(1)=-2.*A1(1)*A1(1)
      IF (T.LT.0.01) THEN
        EX1=T*(1.-T*(0.5-T*(0.1666667-T*0.041666667)))
      ELSE
        IF(T.LT.20.) THEN
          EX1=1.-EXP(-T)
        ELSE
          EX1=1.
        END IF
      END IF
      EX=1.-EX1
      FACT=1.+2.*A1(1)*EX1
      SP2(1)=SP2(1)/FACT
      SP3(1)=SP3(1)/FACT
      IF(ITRAN.LT.10) THEN
        IMINUS(0)=0.0
        P(1)=S(1)
      ELSE
        P(1)=S(1)+2.*A1(1)/FACT*EXP(-T)*IMINUS(0)
      ENDIF
C
C  CALCULATE TRIDIAGONAL COEFFICIENTS
C
      IF(MOD(ITRAN,10).EQ.0) THEN
        DO 200 K=2,NDEP-1
          SP1(K)=-A1(K)
          SP2(K)=1.
          SP3(K)=-C1(K)
          P(K)=S(K)
  200   CONTINUE
      ELSE IF(MOD(ITRAN,10).EQ.1) THEN
        DO 201 K=2,NDEP-1
          AD=.166666666*DTAUQ(K)*2./(DTAUQ(K)+DTAUQ(K+1))
          CD=.166666666*DTAUQ(K+1)*2./(DTAUQ(K)+DTAUQ(K+1))
          SP1(K)=-A1(K)+AD
          SP2(K)=1.
          SP3(K)=-C1(K)+CD
          P(K)=S(K)+AD*(S(K-1)-S(K))+CD*(S(K+1)-S(K))
  201   CONTINUE
      ELSE IF(MOD(ITRAN,10).EQ.2) THEN
        DO 202 K=2,NDEP-1
          AD=.166666666-0.083333333*DTAUQ(K+1)**2/DTAUQ(K)*
     *     2./(DTAUQ(K)+DTAUQ(K+1))
          CD=.166666666-0.083333333*DTAUQ(K)**2/DTAUQ(K+1)*
     *     2./(DTAUQ(K)+DTAUQ(K+1))
          SP1(K)=-A1(K)+AD
          SP2(K)=1.
          SP3(K)=-C1(K)+CD
          P(K)=S(K)+AD*(S(K-1)-S(K))+CD*(S(K+1)-S(K))
  202   CONTINUE
      ELSE
         write(*,*)' TRANF: ITRAN OUTSIDE RANGE'
        STOP
      ENDIF
C
C K=NDEP: LOWER BOUNDARY
C
      SP1(NDEP)=-1.
      SP2(NDEP)=DTAUQ(NDEP)+0.5*DTAUQ(NDEP)**2
      SP3(NDEP)=0.
      P(NDEP)=S(NDEP)*(DTAUQ(NDEP)+0.5*DTAUQ(NDEP)**2)+
     *       (S(NDEP)-S(NDEP-1))
C
C ELIMINATE SUBDIAGONAL
C
      DO 300 K=1,NDEP-1
        F=-SP1(K+1)/(SP2(K)-SP3(K))
        P(K+1)=P(K+1)+F*P(K)
        SP2(K+1)=SP2(K+1)+F*SP2(K)
        SP2(K)=SP2(K)-SP3(K)
  300 CONTINUE
      SP2(NDEP)=SP2(NDEP)-SP3(NDEP)
C
C BACKSUBSTITUTE
C
      P(NDEP)=P(NDEP)/SP2(NDEP)
      DO 320 K=NDEP-1,1,-1
        P(K)=(P(K)-SP3(K)*P(K+1))/SP2(K)
  320 CONTINUE
C
C SURFACE INTENSITY
C
      IPLUS(0)=2.*(EX*P(1)+S(1)*0.5*EX1**2) - IMINUS(0)*EX*EX
C
C  INTENSITIES, CALCULATED FROM A WEIGHTED AVERAGE OF
C  DP/DTAUNY(K+0.5) AND DP/DTAUNY(K-0.5)
C
      IMINUS(1)=EX1*S(1) + EXP(-T)*IMINUS(0)
      IPLUS(1)=2.*P(1)-IMINUS(1)
      DPDT2=(P(2)-P(1))/DTAUQ(2)
      DO 340 K=2,NDEP-1
        DPDT1=DPDT2
        DPDT2=(P(K+1)-P(K))/DTAUQ(K+1)
        PPRIMK=(DTAUQ(K)*DPDT2+DTAUQ(K+1)*DPDT1)/
     *         (DTAUQ(K)+DTAUQ(K+1))
        IPLUS(K)=P(K)+PPRIMK
        IMINUS(K)=P(K)-PPRIMK
  340 CONTINUE
      IPLUS(NDEP)=S(NDEP)+(S(NDEP)-S(NDEP-1))/DTAUQ(NDEP)
      IMINUS(NDEP)=2.0*P(NDEP)-IPLUS(NDEP)
C
      RETURN
      END
      subroutine ng(np,weight,x0,x1,x2,x3)
c
c
c
      include 'prec'
      dimension weight(np),x0(np),x1(np),x2(np),x3(np)
      integer np
c
c
c
      a1=0.
      b1=0.
      a2=0.
      b2=0.
      c1=0.
      c2=0.
      do 1 i=1,np
         wt=weight(i)
         f1=x0(i)-2.*x1(i)+x2(i)
         f2=x0(i)-x1(i)-x2(i)+x3(i)
         f3=x0(i)-x1(i)
         a1=a1+f1*wt*f1
         b1=b1+f2*wt*f1
         a2=b1
         b2=b2+f2*wt*f2
         c1=c1+f1*wt*f3
         c2=c2+f2*wt*f3
 1    continue
      a=(c1*b2-c2*b1)/(a1*b2-a2*b1)
      b=(c2*a1-c1*a2)/(a1*b2-a2*b1)
      do 2,i=1,np
         x3(i)=(1.-a-b)*x0(i)+a*x1(i)+b*x2(i)
 2    continue
 1000 format(1x,i3,5(1p,e10.3),0p)
      return
      end


      SUBROUTINE GAUSI(K,A,B,AI,XI)
C
C  SUPPLIES QUADRATURE WEIGHTS AND POINTS FOR GAUSSIAN QUADRATURE
C  BETWEEN A AND B.   SOURCE OF DATA:    LOWAN,  DAVIDS,
C  LEVENSON,  BULL. AMER. MATH. SOC. 48, P 739 (1942).
C
C  K     : INTEGRATION ORDER, K BETWEEN 1 AND 10 (IN)
C  A     : LOWER QUADRATURE LIMIT                (IN)
C  B     : UPPER QUADRATURE LIMIT                (IN)
C  AI    : QUADRATURE WEIGHTS                   (OUT)
C  XI    : QUADRATURE POINTS                    (OUT)
C
C  CODED BY : B. GUSTAFSSON (1974).
C
      include 'prec'
      DIMENSION AI(K),XI(K)
      DOUBLE PRECISION AP(29),XP(29)
      INTEGER INDOV(9)
      DATA AP/1.0D0
     *,0.55555555555555D0,.88888888888888D0,.347854845137D0
     *,0.65214515486254D0,0.23692688505618D0,0.47862867049936D0,
     * 0.56888888888888D0,0.17132449237917D0,0.36076157304813D0,
     * 0.46791393457269D0,0.12948496616887D0,0.27970539148927D0,
     * 0.38183005050511D0,0.41795918367346D0,0.10122853629037D0,
     * 0.22238103445337D0,0.31370664587788D0,0.36268378337836D0,
     * 0.08127438836157D0,0.18064816069485D0,0.26061069640293D0,
     * 0.31234707704000D0,0.33023935500126D0,0.06667134430868D0,
     * 0.14945134915058D0,0.21908636251598D0,0.26926671930999D0,
     * 0.29552422471475D0/
      DATA XP/
     *0.57735026918962D0,.77459666924148D0,.0D0,0.86113631159405D0,
     *0.33998104358485D0,.90617984593866D0,.53846931010568D0,.0D0,
     *0.93246951420315D0,.66120938646626D0,.23861918608319D0,
     *0.94910791234275D0,.74153118559939D0,.40584515137739D0,.0D0,
     *0.96028985649753D0,.79666647741362D0,.52553240991632D0,
     *0.18343464249565D0,.96816023950762D0,.83603110732663D0,
     *0.61337143270059D0,.32425342340380D0,.0D0,0.97390652851717D0,
     *0.86506336668898D0,.67940956829902D0,.43339539412924D0,
     *0.14887433898163D0/
      DATA INDOV/1,3,5,8,11,15,19,24,29/
C
      IF (K.EQ.1) GO TO 7
C
      KUD=0
      FLK=FLOAT(K)/2.
      K2=K/2
      FK=FLOAT(K2)
      IF (ABS(FLK-FK)-1.E-7) 2,1,1
    1 K2=K2+1
      KUD=1
    2 IOEV=INDOV(K-1)
      INED=IOEV-K2
C
      DO 3 I=1,K2
      IP=INED+I
      XI(I)=-XP(IP)*(B-A)*0.5+(B+A)*0.5
    3 AI(I)=(B-A)*0.5*AP(IP)
C
      K2=K2+1
      DO 4 I=K2,K
      IP=IOEV+K2-I
      IF (KUD) 6,6,5
    5 IP=IP-1
    6 CONTINUE
      XI(I)=XP(IP)*(B-A)*0.5+(B+A)*0.5
    4 AI(I)=(B-A)*0.5*AP(IP)
      RETURN
C
    7 XI(1)=(B+A)*0.5
      AI(1)=B-A
      RETURN
C
      END
      SUBROUTINE VOIGT(A,VS,H)
C
C  COMPUTES A VOIGT FUNCTION  H = H(A,V)
C  A=GAMMA/(4*PI*DNUD)   AND  V=(NU-NU0)/DNUD.  THIS  IS  DONE AFTER
C  TRAVING (LANDOLT-B\RNSTEIN, P. 449).
C
C  CODED BY: KJELL ERIKSSON  (APR-1974). REVISED: (JULY-1975).
C
      DIMENSION AK(19),A1(5)
      DATA AK      /-1.12470432, -0.15516677,  3.28867591, -2.34357915,
     ,  0.42139162, -4.48480194,  9.39456063, -6.61487486,  1.98919585,
     , -0.22041650, 0.554153432, 0.278711796,-0.188325687, 0.042991293,
     ,-0.003278278, 0.979895023,-0.962846325, 0.532770573,-0.122727278/
      DATA SQP/1.772453851/,SQ2/1.414213562/
C
      V = ABS(VS)
      U = A + V
      V2 = V*V
      IF (A.EQ.0.0) GO TO 140
      IF (A.GT.0.2) GO TO 120
      IF (V.GE.5.0) GO TO 121
C
      EX=0.
      IF(V2.LT.100.)EX = EXP(-V2)
      K = 1
C
  100 QUO = 1.
      IF (V.LT.2.4) GO TO 101
      QUO = 1./(V2 - 1.5)
      M = 11
      GO TO 102
C
  101 M = 6
      IF (V.LT.1.3) M = 1
  102 DO 103 I=1,5
         A1(I) = AK(M)
         M = M + 1
  103 CONTINUE
      H1 = QUO*(A1(1) + V*(A1(2) + V*(A1(3) + V*(A1(4) + V*A1(5)))))
      IF (K.GT.1) GO TO 110
C
C A LE 0.2  AND V LT 5.
C
      H = H1*A + EX*(1. + A*A*(1. - 2.*V2))
      RETURN
C
  110 PQS = 2./SQP
      H1P = H1 + PQS*EX
      H2P = PQS*H1P - 2.*V2*EX
      H3P = (PQS*(1. - EX*(1. - 2.*V2)) - 2.*V2*H1P)/3. + PQS*H2P
      H4P = (2.*V2*V2*EX - PQS*H1P)/3. + PQS*H3P
      PSI = AK(16) + A*(AK(17) + A*(AK(18) + A*AK(19)))
C
C 0.2 LT A LE 1.4  AND  A + V LE 3.2
C
      H = PSI*(EX + A*(H1P + A*(H2P + A*(H3P + A*H4P))))
      RETURN
C
  120 IF (A.GT.1.4.OR.U.GT.3.2) GO TO 130
      EX=0.
      IF(V2.LT.100.)EX = EXP(-V2)
      K = 2
      GO TO 100
C
C A LE 0.2  AND  V GE 5.
C
  121 H = A*(15. + 6.*V2 + 4.*V2*V2)/(4.*V2*V2*V2*SQP)
      RETURN
C
  130 A2 = A*A
      U = SQ2*(A2 + V2)
      U2 = 1./(U*U)
C
C A GT 1.4  OR  A + V GT 3.2
C
      H = SQ2/SQP*A/U*(1. + U2*(3.*V2 - A2) +
     ,        U2*U2*(15.*V2*V2 - 30.*V2*A2 + 3.*A2*A2))
      RETURN
C
C A EQ 0.
C
  140 H=0.
      IF(V2.LT.100.)H=EXP(-V2)
      RETURN
      END
C
C********************************************************************
C
