C
C*********************************************************************
C
C
C  Note from Phil to Students:
C
C  This might even be self explanatory: but there are some conventions
C  to follow
C
C  1.  Let your depth grid start at point 1 to ndep: 1 is the outermost
C      point.  This is important since the boundary conditions are 
C      written with this explicitly in mind.
C  2.  Should you want to include incoming radiation, use IMINUS(0)=...
C      and a suitable choice of ITRAN
C  3.  The variable P that is output here is variable u in the Olson 
C      et al paper. 
C
C  4.  You should just call the routine TRANSP to get a formal solution
C      (it calls the routine TRANF itself).
C
C  5.  Yes, Daniel found the compilation error in tranf.f -
C      this has been fixed May 11, 2000
C  FYI:
C  My calling program that solves the OAB problem based on the attached 
C  routines contains a total of 180 lines of code- 
C
C  the actual "guts" of the thing
C  is less than 100 lines of code
C
C*********************************************************************
C
      SUBROUTINE TRANSP(XMY,NDEP,TAU,S,ITRAN,
     *                  P,PMS,IPLUS,IMINUS)
C
C  SOLVES THE RADIATIVE TRANSFER EQUATION WITH GIVEN SOURCE FUNCTION.
C  SOLVES FOR P-S WHERE P IS FEAUTRIERS P; I.E.
C
C    P = 0.5*(I(XMU)+I(-XMU))
C
C  IN THE PRESENCE OF VELOCITY FIELDS:
C
C    P = 0.5*(I(NY,XMU) + I(-NY,-XMU))
C
C  ITRAN DETERMINES THE MODE OF FORMAL SOLUTION
C  ITRAN  = 0  FEAUTRIER SOLUTION
C           1  FEAUTRIER SOLUTION TO CUBIC SPLINE ACCURACY, REF:
C              KUNASZ, HUMMER, 1974, MNRAS 166,19
C              MIHALAS, 1974, APJ SUPPL 28,343
C           2  FEAUTRIER SOLUTION HERMITE, REF:
C              AUER, 1976, JQSRT 16,931
C
C  VARIABLES:
C
C  XMY   : ANGULAR COSINE                                       (IN)
C  NDEP  : NUMBER OF DEPTH POINTS                               (IN)
C  TAU   : MONOCHROMATIC OPTICAL DEPTH SCALE                    (IN)
C  S     : MONOCHROMATIC SOURCE FUNCTION                        (IN)
C  ITRAN : DETERMINES MODE OF FORMAL SOLUTION                   (IN)
C
C  P     : MEAN BIDIRECTIONAL INTENSITY (CF. ABOVE)            (OUT)
C  PMS   : P-S                                                 (OUT)
C  IPLUS : RADIATION INTENSITY, OUTGOING RAYS                  (OUT)
C  IMINUS: RADIATION INTENSITY, INGOING RAYS                   (IN/OUT)
C
C:
C: TRANSP 86-09-04  MODIFICATIONS: (GORAN SCHARMER, MATS CARLSSON)
C:        INCLUDES CALL TO CUBIC SPLINE INTEGRAL FORMAL SOLVER
C:
C:        92-10-08  MODIFICATIONS: (MATS CARLSSON)
C:        ITRAN=10-14 ALLOWED (ITRAN.GT.10 INCLUDES INCOMING RADIATION)
C:
      include 'prec'
      parameter (mdep=200)
      dimension tau(ndep), s(ndep)
      dimension p(ndep), pms(ndep), iplus(0:ndep),iminus(0:ndep)
      REAL*8 IPLUS,IMINUS
      integer ITRAN
      dimension TAUQ(0:MDEP),DTAUQ(MDEP),A1(MDEP),C1(MDEP)
C
C
C 
      IF(NDEP .GT. MDEP) THEN
         WRITE(6,*)' YOU MUST INCREASE MDEP TO BE .GE. NDEP'
         STOP
      ENDIF
C
C K=1: UPPER BOUNDARY
C
      CMU=0.5/XMY
      DTAUQ(2)=(2.)*(TAU(2)-TAU(1))*CMU
      A1(1)=1./DTAUQ(2)
      T=TAU(1)*1.0*2.0*CMU
      TAUQ(1)=T
      DTAUQ(1)=T
C
C  CALCULATE DTAUQ 
C
      DO 100 K=2,NDEP
        DTAUQ(K)=2.0*(TAU(K)-TAU(K-1))*CMU
  100 CONTINUE
C
C  CALCULATE TAUQ
C
      DO 110 K=2,NDEP
        TAUQ(K)=TAUQ(K-1)+DTAUQ(K)
  110 CONTINUE
C
C  CALCULATE A1 AND C1
C
      DO 120 K=2,NDEP-1
        A1(K)=2./(DTAUQ(K)+DTAUQ(K+1))/DTAUQ(K)
        C1(K)=2./(DTAUQ(K)+DTAUQ(K+1))/DTAUQ(K+1)
  120 CONTINUE
C
C  CHOOSE THE METHOD OF FORMAL SOLUTION
C
      IF(MOD(ITRAN,10).LE.2) THEN
        CALL TRANF(itran,ndep,tauq,dtauq,s,a1,c1,p,iplus,iminus)
      ELSE
        write(6,*)'MOD(ITRAN,10) MUST BE 1,2,11, or 12'
        stop
      ENDIF
C
C CALCULATE P(K)-S(K)
C
      PMS(1)=P(1)-S(1)
      DO 130 K=2,NDEP-1
        IF(ABS(A1(K)).GT.1.0) THEN
          PMS(K)=P(K)-S(K)
        ELSE
          PMS(K)=C1(K)*(P(K+1)-P(K))-A1(K)*(P(K)-P(K-1))
        END IF
  130 CONTINUE
      PMS(NDEP)=(P(NDEP-1)-P(NDEP)+S(NDEP)-S(NDEP-1))
     * /(DTAUQ(NDEP)+0.5*DTAUQ(NDEP)**2)
C
c      do 150  k=1,ndep
c         write(6,*) k,'P,pms ',p(k),pms(k)
c 150  continue
      RETURN
      END
C
C**********************************************************************
C
      SUBROUTINE TRANF(itran,ndep,tauq,dtauq,s,a1,c1,p,iplus,iminus)
C
C  FORMAL SOLVER FOR THE TRANSFER EQUATION USING FEAUTRIER TECHNIQUE
C
C  MOD(ITRAN,10) DETERMINES THE MODE OF FORMAL SOLUTION
C        =  0  FEAUTRIER SOLUTION
C           1  FEAUTRIER SOLUTION TO CUBIC SPLINE ACCURACY, REF:
C              AUER, 1976, JQSRT 16,931
C           2  FEAUTRIER SOLUTION HERMITE
C
C  THE  UPPER AND LOWER  BOUNDARY CONDITIONS ARE DERIVED FROM SECOND
C  ORDER TAYLOR EXPANSIONS, WITH ESTIMATES OF THE INCIDENT RADIATION
C  FIELDS:
C
C    P(2)    =  P(1) + DTAU*P'(1) + 0.5*DTAU**2*P''(1)
C    P'(1)   =  P(1) - I(-XMU)
C    I(-XMU) =  S(1)*(1.-EXP(-TAUQ(1)))
C
C    P(N-1)  =  P(N) - DTAU*P'(N) + 0.5*DTAU**2*P''(N)
C    P'(N)   =  I(XMU) - P(N)
C    I(XMU)  =  S(N) + (S(N)-S(N-1))/DTAU
C
C  STEINS  TRICK
C  (STORING THE SUM OF ELEMENTS INSTEAD OF THE DIAGONAL) IS USED  TO
C  AVOID  NUMERICAL  ROUND-OFF  PROBLEMS  AT  SMALL  OPTICAL DEPTHS.
C
C  CODED BY: A.NORDLUND (OCT-1981). REVISED (MAR-1982).
C
C: TRANF  92-10-08  MODIFICATIONS: (MATS CARLSSON)
C:        INCOMING RADIATION FIELD IS SPECIFIED IN IMINUS(0)
C:        THIS IS INDICATED BY ITRAN.GT.10
C:
      include 'prec'
      parameter (mdep=200)
      dimension s(ndep)
      dimension p(ndep), iplus(0:ndep),iminus(0:ndep)
      REAL*8 IPLUS,IMINUS
      integer ITRAN
      dimension TAUQ(0:MDEP),DTAUQ(MDEP),A1(MDEP),C1(MDEP)
C
      DIMENSION SP1(MDEP),SP2(MDEP),SP3(MDEP)
C
      T=TAUQ(1)
      SP1(1)=0.
      SP2(1)=1.+2.*A1(1)
      SP3(1)=-2.*A1(1)*A1(1)
      IF (T.LT.0.01) THEN
        EX1=T*(1.-T*(0.5-T*(0.1666667-T*0.041666667)))
      ELSE
        IF(T.LT.20.) THEN
          EX1=1.-EXP(-T)
        ELSE
          EX1=1.
        END IF
      END IF
      EX=1.-EX1
      FACT=1.+2.*A1(1)*EX1
      SP2(1)=SP2(1)/FACT
      SP3(1)=SP3(1)/FACT
      IF(ITRAN.LT.10) THEN
        IMINUS(0)=0.0
        P(1)=S(1)
      ELSE
        P(1)=S(1)+2.*A1(1)/FACT*EXP(-T)*IMINUS(0)
      ENDIF
C
C  CALCULATE TRIDIAGONAL COEFFICIENTS
C
      IF(MOD(ITRAN,10).EQ.0) THEN
        DO 200 K=2,NDEP-1
          SP1(K)=-A1(K)
          SP2(K)=1.
          SP3(K)=-C1(K)
          P(K)=S(K)
  200   CONTINUE
      ELSE IF(MOD(ITRAN,10).EQ.1) THEN
        DO 201 K=2,NDEP-1
          AD=.166666666*DTAUQ(K)*2./(DTAUQ(K)+DTAUQ(K+1))
          CD=.166666666*DTAUQ(K+1)*2./(DTAUQ(K)+DTAUQ(K+1))
          SP1(K)=-A1(K)+AD
          SP2(K)=1.
          SP3(K)=-C1(K)+CD
          P(K)=S(K)+AD*(S(K-1)-S(K))+CD*(S(K+1)-S(K))
  201   CONTINUE
      ELSE IF(MOD(ITRAN,10).EQ.2) THEN
        DO 202 K=2,NDEP-1
          AD=.166666666-0.083333333*DTAUQ(K+1)**2/DTAUQ(K)*
     *     2./(DTAUQ(K)+DTAUQ(K+1))
          CD=.166666666-0.083333333*DTAUQ(K)**2/DTAUQ(K+1)*
     *     2./(DTAUQ(K)+DTAUQ(K+1))
          SP1(K)=-A1(K)+AD
          SP2(K)=1.
          SP3(K)=-C1(K)+CD
          P(K)=S(K)+AD*(S(K-1)-S(K))+CD*(S(K+1)-S(K))
  202   CONTINUE
      ELSE
         write(*,*)' TRANF: ITRAN OUTSIDE RANGE'
        STOP
      ENDIF
C
C K=NDEP: LOWER BOUNDARY
C
      SP1(NDEP)=-1.
      SP2(NDEP)=DTAUQ(NDEP)+0.5*DTAUQ(NDEP)**2
      SP3(NDEP)=0.
      P(NDEP)=S(NDEP)*(DTAUQ(NDEP)+0.5*DTAUQ(NDEP)**2)+
     *       (S(NDEP)-S(NDEP-1))
C
C ELIMINATE SUBDIAGONAL
C
      DO 300 K=1,NDEP-1
        F=-SP1(K+1)/(SP2(K)-SP3(K))
        P(K+1)=P(K+1)+F*P(K)
        SP2(K+1)=SP2(K+1)+F*SP2(K)
        SP2(K)=SP2(K)-SP3(K)
  300 CONTINUE
      SP2(NDEP)=SP2(NDEP)-SP3(NDEP)
C
C BACKSUBSTITUTE
C
      P(NDEP)=P(NDEP)/SP2(NDEP)
      DO 320 K=NDEP-1,1,-1
        P(K)=(P(K)-SP3(K)*P(K+1))/SP2(K)
  320 CONTINUE
C
C SURFACE INTENSITY
C
      IPLUS(0)=2.*(EX*P(1)+S(1)*0.5*EX1**2) - IMINUS(0)*EX*EX
C
C  INTENSITIES, CALCULATED FROM A WEIGHTED AVERAGE OF
C  DP/DTAUNY(K+0.5) AND DP/DTAUNY(K-0.5)
C
      IMINUS(1)=EX1*S(1) + EXP(-T)*IMINUS(0)
      IPLUS(1)=2.*P(1)-IMINUS(1)
      DPDT2=(P(2)-P(1))/DTAUQ(2)
      DO 340 K=2,NDEP-1
        DPDT1=DPDT2
        DPDT2=(P(K+1)-P(K))/DTAUQ(K+1)
        PPRIMK=(DTAUQ(K)*DPDT2+DTAUQ(K+1)*DPDT1)/
     *         (DTAUQ(K)+DTAUQ(K+1))
        IPLUS(K)=P(K)+PPRIMK
        IMINUS(K)=P(K)-PPRIMK
  340 CONTINUE
      IPLUS(NDEP)=S(NDEP)+(S(NDEP)-S(NDEP-1))/DTAUQ(NDEP)
      IMINUS(NDEP)=2.0*P(NDEP)-IPLUS(NDEP)
C
      RETURN
      END
